import * as HardSourceWebpackPlugin from 'hard-source-webpack-plugin';
import * as webpack from 'webpack';
import ForkTsCheckerWebpackPlugin = require('fork-ts-checker-webpack-plugin');

const TS_LOADER = [
    { loader: 'cache-loader', options: { cacheDirectory: "node_modules/.cache_loader" } },
    {
        loader: 'ts-loader',
        options: {
            transpileOnly: true,
            experimentalWatchApi: true,
        },
    }
];

const config: webpack.Configuration = {
    devtool: "source-map",
    //项目需要解析的文件拓展名称
    resolve: {
        extensions: [".ts", ".tsx", ".js", "json"]
    },
    //模块加载器设置
    module: {
        rules: [
            {
                test: /\.tsx?$/,
                exclude: /node_modules/,
                use: TS_LOADER,
            },
            { test: /\.css$/, loader: ['style-loader', 'css-loader'] },
            { test: /\.[(jpg)|(png)|(obj)|(json)]$/, loader: "url-loader" },
        ]
    },

    plugins: [
        new HardSourceWebpackPlugin(),
        new ForkTsCheckerWebpackPlugin({ checkSyntacticErrors: true }),
    ]
}

export default config;
